// C Source File
// Created 1/26/2004; 17.15.12

#ifndef _UTILS_C_
#define _UTILS_C_

#include "utils.h"

void isWinPossible(char *done, Grid *grid, const short *size, const Wall *walls, const short *numWalls, const short startR, const short startC, const char *endR, const char *endC)
{
	if ((startR == *endR || *endR == -1) && 
			(startC == *endC || *endC == -1))
	{
		*done = 1;
	}

	if (*done == 1)
		return;
	
	grid->data[startR][startC] = 1;
	//DrawLine(startC*CELL_WIDTH+BOARDOFFSET_X, startR*CELL_WIDTH+BOARDOFFSET_Y, startC*CELL_WIDTH+BOARDOFFSET_X+CELL_WIDTH, startR*CELL_WIDTH+BOARDOFFSET_Y+CELL_WIDTH, A_NORMAL);
	
	
	if (testMove(walls, *numWalls, *size, startR, startC, 1, 0) && grid->data[startR+1][startC] == 0)
		isWinPossible(done, grid, size, walls, numWalls, startR+1, startC, endR, endC);

	if (testMove(walls, *numWalls, *size, startR, startC, 0, 1) && grid->data[startR][startC+1] == 0)
		isWinPossible(done, grid, size, walls, numWalls, startR, startC+1, endR, endC);

	if (testMove(walls, *numWalls, *size, startR, startC, -1, 0) && grid->data[startR-1][startC] == 0)
		isWinPossible(done, grid, size, walls, numWalls, startR-1, startC, endR, endC);

	if (testMove(walls, *numWalls, *size, startR, startC, 0, -1) && grid->data[startR][startC-1] == 0)
		isWinPossible(done, grid, size, walls, numWalls, startR, startC-1, endR, endC);
	
}

int movePlayer(Player *player, char numPlayers, short index, short size, Wall *walls, int numWalls, char moveR, char moveC)
{
	int newlocR = player[index].r + moveR;
	int	newlocC = player[index].c + moveC;

	//first check out of bounds errors	
	if (newlocR < 0 || newlocC < 0 || newlocR >= size || newlocC >= size) {
		sendMessage("Movement out of bounds");
		return 1;
	}
	
	int i;
	
	for (i=0; i<numPlayers; i++)
		if (newlocR == player[i].r && newlocC == player[i].c && player[i].hasWon == 0) {
			sendMessage("Movement blocked by other pawn");
			return 1;
		}
		
	for (i=0; i<numWalls; i++)
	{
		if (moveR != 0 && walls[i].dir == HORIZONTAL) {
			if ((player[index].c == walls[i].c-1 || player[index].c == walls[i].c) 
			&& ((newlocR == walls[i].r && player[index].r == walls[i].r-1) || (newlocR == walls[i].r-1 && player[index].r == walls[i].r))) {
					sendMessage("Movement blocked by wall");
					return 1;
				}
		}
		
		if (moveC != 0 && walls[i].dir == VERTICAL) {
			if ((player[index].r == walls[i].r-1 || player[index].r == walls[i].r) 
			&& ((newlocC == walls[i].c && player[index].c == walls[i].c-1) || (newlocC == walls[i].c-1 && player[index].c == walls[i].c))) {
					sendMessage("Movement blocked by wall");
					return 1;
				}
		}
	}
	
		
	player[index].r = newlocR;
	player[index].c = newlocC;
	
	return 0;
}

short mainmenu(short *numPlayers)
{
	int centerX = LCD_WIDTH/2;
	int centerY = LCD_HEIGHT/2;
	
	char buffer[32];
	
	FontSetSys(F_6x8);
	
	sprintf(buffer, "<-> %d Human Players", *numPlayers);
	SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);
	SetPlane(1); DrawStr(centerX-60, centerY+28, "    2 Link Players", A_REPLACE);
	SetPlane(1); DrawStr(centerX-60, centerY+36, "    Options", A_REPLACE);

	int done = 0;
	
	int selection = 0;

	int key, i;
	
	while(!done)
	{
		SetPlane(0);
		for (i=0; i<8; i++)
			DrawLine(centerX-60, centerY+20+i+selection*8, centerX+60, centerY+20+i+selection*8, A_XOR);

		key = ngetchx();

		SetPlane(0);
		for (i=0; i<8; i++)
			DrawLine(centerX-60, centerY+20+i+selection*8, centerX+60, centerY+20+i+selection*8, A_XOR);

		if (key == KEY_ENTER && selection == 0)
			done = 1;
			
		if (key == KEY_ENTER && selection == 1)
			done = 2;
			
		if (key == KEY_ENTER && selection == 2)
			done = 3;
			
		if (key == KEY_ESC)
			return 0;
			
		if (selection == 0 && key == KEY_LEFT && *numPlayers > 2)
		{
			*numPlayers = *numPlayers - 1;
			sprintf(buffer, "<-> %d human players", *numPlayers);
			SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);
		}

		if (selection == 0 && key == KEY_RIGHT && *numPlayers < 4)
		{
			*numPlayers = *numPlayers + 1;
			sprintf(buffer, "<-> %d human players", *numPlayers);
			SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);
		}
		
		if (key == KEY_UP && selection > 0)
			selection--;
			
		if (key == KEY_DOWN && selection < 2)
			selection++;
			
	}
	
	return done;
}

short optionsmenu(short *boardsize)
{
	int centerX = LCD_WIDTH/2;
	int centerY = LCD_HEIGHT/2;
	
	char buffer[32];
	
	FontSetSys(F_6x8);
	
	sprintf(buffer, "<-> %d Size Grid  ", *boardsize);
	SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);

	int done = 0;
	
	int selection = 0;

	int key, i;
	
	while(!done)
	{
		SetPlane(0);
		for (i=0; i<8; i++)
			DrawLine(centerX-60, centerY+20+i+selection*8, centerX+60, centerY+20+i+selection*8, A_XOR);

		key = ngetchx();

		SetPlane(0);
		for (i=0; i<8; i++)
			DrawLine(centerX-60, centerY+20+i+selection*8, centerX+60, centerY+20+i+selection*8, A_XOR);

		if (key == KEY_ESC || key == KEY_ENTER)
			return 0;
			
		if (selection == 0 && key == KEY_LEFT && *boardsize > 5)
		{
			*boardsize = *boardsize - 2;
			sprintf(buffer, "<-> %d Size Grid  ", *boardsize);
			SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);
		}

		if (selection == 0 && key == KEY_RIGHT && *boardsize < 11)
		{
			*boardsize = *boardsize + 2;
			sprintf(buffer, "<-> %d Size Grid  ", *boardsize);
			SetPlane(1); DrawStr(centerX-60, centerY+20, buffer, A_REPLACE);
		}
		
		if (key == KEY_UP && selection > 0)
			selection--;
			
		if (key == KEY_DOWN && selection < 0)
			selection++;
			
	}
	
	return done;
}

#endif
